@@ -1,4 +1,4 @@
-use super::{input::ParserInput, Ctx, Parser};
+use super::{input::ParserInput, Ctx, PResult, Parse, Parser};
 use std::ops::{Deref, DerefMut};
 use swc_common::Span;
 
@@ -22,6 +22,14 @@ where
             inner: self,
         }
     }
+
+    #[inline]
+    pub(super) fn parse_as<T>(&mut self) -> PResult<T>
+    where
+        Self: Parse<T>,
+    {
+        self.parse()
+    }
 }
 
 pub(super) struct WithCtx<'w, I: 'w + ParserInput> {
